from typing import Dict

class Component: 

   

    def __init__(self, id: str, attributes: Dict) -> None:
        self.id = id
        self.attributes = attributes
        self.__create_styles(attributes.get('style'))
        
    def __create_styles(self, style_list: str):
        self.styles = {}
        if not style_list == None:
            self.styles = style_list.split(";")[-1]
            
    def get_styles(self):
        return self.styles

    def get_value(self):
        return self.attributes.get('value')
    
    def get_id(self):
        return self.id
    
    def get_component_status(self):
        return self.styles.get('cvc-component-status')

    def get_lib_version(self):
        return self.styles.get('cvc-lib-version')

    def get_parent(self):
        return self.attributes.get('parent')
    
    def get_icon_id(self):
        return self.styles.get('cvc-icon-id')
    
    def get_architecture_layer(self):
        return self.styles.get('cvc-architecture-layer')

    def get_landscape_layer(self):
        return self.styles.get('cvc-landscape-layer')

    def get_technology(self):
        return self.styles.get('cvc-technology')

    def get_technology_framework(self):
        return self.styles.get('cvc-technology-framework')   

    def get_template_type(self):
        return self.styles.get('cvc-template-type')

    def get_template_name(self):
        return self.styles.get('cvc-template-name')



  


from typing import List


class Diagram:
    def __init__(self, name: str, components: List) -> None:
        self.name = name
        self.__components = components
    
    def get_components(self):
        return self.__components


import base64
import zlib
import xml.etree.ElementTree as ET
import urllib.parse

from drawio_parser.component import Component
from drawio_parser.diagram import Diagram

class Parser:

    def __init__(self, drawio_file) -> None:
        self.__diagrams = []
        self.__parse_drawio(ET.parse(drawio_file))

    def __extract_diagram_content(self, diagram) -> str:
        diagram_content: str = diagram.text or ""
        diagram_content_decoded: bytes = base64.b64decode(diagram_content)
        xml_content: bytes = zlib.decompress(diagram_content_decoded, -15)

        return urllib.parse.unquote(xml_content.decode())

    def __parse_drawio(self, drawio_file):
        
        for diagram in drawio_file.findall("diagram"):
            diagram_name: str = diagram.get("name", "")

            xml_content_str: str = self.__extract_diagram_content(diagram)
            
            xml_content: ET.Element = ET.fromstring(xml_content_str)
            xml_root = xml_content.findall('root')[0]
            diagram_elements = xml_root.findall('mxCell')
            components = []
            for e in diagram_elements:
                component = Component(e.get('id'), dict(e.items()))
                components.append(component)
            diagram = Diagram(diagram_name, components)
            self.__diagrams.append(diagram)

    def get_diagrams(self):
        return self.__diagrams
